@@ -25,16 +25,15 @@ import re
 import shlex
 import subprocess
 import sys
-from xml.etree import ElementTree as ET
-from utils import (get_nmap_path
-)
+from nmap3.utils import get_nmap_path # from utils import get_nmap_path
+from utils import get_nmap_path
 import simplejson as json
 import argparse
-import nmapparser
+from nmap3.nmapparser import NmapCommandParser #from nmapparser import NmapCommandParser
 
 __author__ = 'Wangolo Joel (info@nmapper.com)'
 __version__ = '0.1.1'
-__last_modification__ = '2019/11/22'
+__last_modification__ = '2020/04/24'
 
 class Nmap(object):
     """
@@ -53,6 +52,7 @@ class Nmap(object):
         self.maxport = 65389
         self.host = ""
         self.top_ports = dict()
+        self.parser  = NmapCommandParser(None)
 
     def default_command(self):
         """
@@ -62,7 +62,7 @@ class Nmap(object):
         """
         return self.default_args.format(nmap=self.nmaptool, outarg="-oX")
 
-    def scan_top_ports(self, host, default=10):
+    def scan_top_ports(self, host, default=10, args=None):
         """
         Perform nmap's top ports scan
 
@@ -71,29 +71,35 @@ class Nmap(object):
 
         This top port requires root previledges
         """
+        parser  = NmapCommandParser(None)
 
         if(default > self.maxport):
             raise ValueError("Port can not be greater than default 65389")
         self.host = host
 
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
         top_port_args = " {host} --top-ports {default}".format(host=host, default=default)
-        top_port_command = self.default_command() + top_port_args
-        top_port_shlex = shlex.split(top_port_command) # prepare it for popen
-        
+        scan_command = self.default_command() + top_port_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
+
         # Run the command and get the output
-        output = self.run_command(top_port_shlex)
+        output = self.run_command(scan_shlex)
         if not output:
             # Probaby and error was raise
             raise ValueError("Unable to perform requested command")
 
         # Begin parsing the xml response
         xml_root = self.get_xml_et(output)
-        self.top_ports = self.filter_top_ports(xml_root)
+        self.top_ports = parser.filter_top_ports(xml_root)
         return self.top_ports
 
     def nmap_dns_brute_script(self, host, dns_brute="--script dns-brute.nse"):
         """
-        Perform nmap scan usign the dns-brute script
+        Perform nmap scan using the dns-brute script
 
         :param: host can be IP or domain
         :param: default is the default top port
@@ -101,6 +107,7 @@ class Nmap(object):
         nmap -oX - nmmapper.com --script dns-brute.nse
         """
         self.host = host
+        parser  = NmapCommandParser(None)
 
         dns_brute_args = "{host}  {default}".format(host=host, default=dns_brute)
 
@@ -112,12 +119,12 @@ class Nmap(object):
 
         # Begin parsing the xml response
         xml_root = self.get_xml_et(output)
-        subdomains = self.filter_subdomains(xml_root)
+        subdomains = parser.filter_subdomains(xml_root)
         return subdomains
 
-    def nmap_version_detection(self, host, arg="-sV"):
+    def nmap_version_detection(self, host, arg="-sV", args=None):
         """
-        Perform nmap scan usign the dns-brute script
+        Perform nmap scan using the dns-brute script
 
         :param: host can be IP or domain
 
@@ -126,101 +133,107 @@ class Nmap(object):
         self.host = host
 
         command_args = "{host}  {default}".format(host=host, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
 
-        # Run the command and get the output
-        output = self.run_command(dns_brute_shlex)
+        output = self.run_command(scan_shlex)
         xml_root = self.get_xml_et(output)
 
-        services = self.version_parser(xml_root)
+        services = self.parser.version_parser(xml_root)
         return services
-        
-    def nmap_stealth_scan(self, host, arg="-sA"):
+
+    def nmap_stealth_scan(self, host, arg="-sA", args=None):
         """
-        nmap -oX - nmmapper.com --script dns-brute.nse
+        nmap -oX - nmmapper.com -sA
         """
         # TODO
         self.host = host
 
         command_args = "{host}  {default}".format(host=host, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
 
         # Run the command and get the output
-        output = self.run_command(dns_brute_shlex)
+        output = self.run_command(scan_shlex)
         xml_root = self.get_xml_et(output)
-        
-    def nmap_detect_firewall(self, host, arg="-sA"): # requires root
+
+    def nmap_detect_firewall(self, host, arg="-sA", args=None): # requires root
         """
-        nmap -oX - nmmapper.com --script dns-brute.nse
+        nmap -oX - nmmapper.com -sA
+        @ TODO
         """
         self.host = host
 
         command_args = "{host}  {default}".format(host=host, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
-        
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
         # TODO
-        
-        # Run the command and get the output
-        output = self.run_command(dns_brute_shlex)
-        xml_root = self.get_xml_et(output)
 
-    def nmap_os_detection(self, host, arg="-O"): # requires root
+    def nmap_os_detection(self, host, arg="-O", args=None): # requires root
         """
-        nmap -oX - nmmapper.com --script dns-brute.nse
+        nmap -oX - nmmapper.com -O
         NOTE: Requires root
         """
         self.host = host
 
         command_args = "{host}  {default}".format(host=host, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
- 
-        output = self.run_command(dns_brute_shlex)
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
+
+        output = self.run_command(scan_shlex)
         xml_root = self.get_xml_et(output)
-        
-        os_identified = self.os_identifier_parser(xml_root)
+
+        os_identified = self.parser.os_identifier_parser(xml_root)
         return os_identified
-        
-    def nmap_subnet_scan(self, subnet, arg="-p-"): # requires root
+
+    def nmap_subnet_scan(self, subnet, arg="-p-", args=None): # requires root
         """
-        nmap -oX - nmmapper.com --script dns-brute.nse
+        nmap -oX - nmmapper.com -p-
         NOTE: Requires root
         """
         self.host = subnet
-        parser  = nmapparser.NmapCommandParser(None)
-        
+
         command_args = "{host}  {default}".format(host=subnet, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
-        output = self.run_command(dns_brute_shlex)
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+
+        scan_shlex = shlex.split(scan_command)
+        output = self.run_command(scan_shlex)
         xml_root = self.get_xml_et(output)
-        
-        host_discovered = parser.parse_nmap_subnetscan(xml_root)
+
+        host_discovered = self.parser.parse_nmap_subnetscan(xml_root)
         return host_discovered
-        
-    def nmap_list_scan(self, subnet, arg="-sL"): # requires root
+
+    def nmap_list_scan(self, subnet, arg="-sL", args=None): # requires root
         """
-        The list scan is a degenerate form of host discovery that simply lists each host of the network(s) 
+        The list scan is a degenerate form of host discovery that simply lists each host of the network(s)
         specified, without sending any packets to the target hosts.
-        
+
         NOTE: /usr/bin/nmap  -oX  -  192.168.178.1/24  -sL
         """
         self.host = subnet
-        parser  = nmapparser.NmapCommandParser(None)
-        
+
         command_args = "{host}  {default}".format(host=subnet, default=arg)
-        command = self.default_command() + command_args
-        dns_brute_shlex = shlex.split(command) # prepare it for popen
-             
-        output = self.run_command(dns_brute_shlex)
+        scan_command = self.default_command() + command_args
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command)
+
+        output = self.run_command(scan_shlex)
         xml_root = self.get_xml_et(output)
-        
-        host_discovered = parser.parse_nmap_listscan(xml_root)
+
+        host_discovered = self.parser.parse_nmap_listscan(xml_root)
         return host_discovered
-        
+
     def run_command(self, cmd):
         """
         Runs the nmap command using popen
@@ -243,222 +256,268 @@ class Nmap(object):
         """
         return ET.fromstring(command_output)
 
-    def filter_top_ports(self, xmlroot):
-        """
-        Given the xmlroot return the all the ports that are open from
-        that tree
-        """
-        try:
-            port_results = []
-
-            scanned_host = xmlroot.find("host")
-            if(scanned_host):
-                ports = scanned_host.find("ports").findall("port")
-
-                # slowly parse what is required
-                for port in ports:
-                    open_ports = {}
-
-                    open_ports["protocol"]=port.attrib.get("protocol")
-                    open_ports["port"]=port.attrib.get("portid")
-
-                    if port.find('state') != None:
-                        open_ports["state"]=port.find("state").attrib.get("state")
-                        open_ports["reason"]=port.find('state').attrib.get("reason")
-                        open_ports["reason_ttl"]=port.find("state").attrib.get("reason_ttl")
-
-                    if  port.find("service") != None:
-                        open_ports["service"]=port.find("service").attrib
-
-                    port_results.append(open_ports)
-
-        except Exception as e:
-            raise(e)
-        else:
-            return port_results
-
-    def version_parser(self, xmlroot):
-        """
-        Parse version detected
-        """
-        try:
-            service_version = []
-
-            host = xmlroot.find("host")
-            if(host):
-                ports  = host.find("ports")
-                port_service = None
-
-                if(ports):
-                    port_service = ports.findall("port")
-
-                if(port_service):
-                    for port in port_service:
-                        service = {}
-
-                        service["protocol"]=port.attrib.get("protocol")
-                        service["port"]=port.attrib.get("portid")
-
-                        if(port.find("state")):
-                            for s in port.find("state").attrib:
-                                service[s]=port.find("state").attrib.get(s)
-
-                        if(port.find("service")):
-                            service["service"]=port.find("service").attrib
-                            
-                            for cp in port.find("service").findall("cpe"):
-                                cpe_list = []
-                                cpe_list.append({"cpe":cp.text})
-                                service["cpe"]=cpe_list
-                                
-                        service_version.append(service)
-                        
-        except Exception as e:
-            raise(e)
-        else:
-            return service_version
-            
-    def os_identifier_parser(self, xmlroot):
-        """
-        Parser for identified os
-        """
-        try:
-            os_identified = []
-
-            host = xmlroot.find("host")
-            if(host):
-                os = host.find("os")
-                
-                if(host):
-                    osmatches = os.findall("osmatch")
-                    
-                    for match in osmatches:
-                        attrib = match.attrib
-                        
-                        for osclass in match.findall("osclass"):
-                            attrib["osclass"]=osclass.attrib
-                            
-                            for cpe in osclass.findall("cpe"):
-                                attrib["cpe"]=cpe.text 
-                                
-                        os_identified.append(attrib)
-                    
-        except Exception as e:
-            raise(e)
-        else:
-            return os_identified
-
 class NmapScanTechniques(Nmap):
     """
     Extends Nmap to include nmap commands
     with different scan techniques
-    
+
     This scan techniques include
-    
+
     1) TCP SYN Scan (-sS)
     2) TCP connect() scan (-sT)
     3) FIN Scan (-sF)
     4) Ping Scan (-sP)
     5) Idle Scan (-sI)
     """
-    
+
     def __init__(self, path=None):
         super(NmapScanTechniques, self).__init__(path=path)
-        
+
         self.sync_scan = "-sS"
         self.tcp_connt = "-sT"
         self.fin_scan = "-sF"
         self.ping_scan = "-sP"
         self.idle_scan = "-sL"
-    
-    def nmap_fin_scan(self, host):
+        self.udp_scan = "-sU"
+
+    def nmap_fin_scan(self, host, args=None):
         """
         Perform scan using nmap's fin scan
-        
+
         @cmd nmap -sF 192.168.178.1
+
         """
         fin_scan = " {host} {default}".format(host=host, default=self.fin_scan)
         fin_scan_command = self.default_command() + fin_scan
-        fin_scan_shlex = shlex.split(fin_scan_command) # prepare it 
-        parser  = nmapparser.NmapCommandParser(None)
-        
+        if(args):
+            fin_scan_command += " {0}".format(args)
+        fin_scan_shlex = shlex.split(fin_scan_command)
+        parser  = NmapCommandParser(None)
+
         # Use the ping scan parser
         output = self.run_command(fin_scan_shlex)
         xml_root = self.get_xml_et(output)
         fin_results = parser.parse_nmap_idlescan(xml_root)
         return fin_results
-        
-    def nmap_syn_scan(self, host):
+
+    def nmap_syn_scan(self, host, args=None):
         """
         Perform syn scan on this given
         host
-        
+
         @cmd nmap -sS 192.168.178.1
         """
+        parser  = NmapCommandParser(None)
+
         sync_scan = " {host} {default}".format(host=host, default=self.sync_scan)
         sync_scan_command = self.default_command() + sync_scan
-        sync_scan_shlex = shlex.split(sync_scan_command) # prepare it 
-        
+        if(args):
+            sync_scan_command += " {0}".format(args)
+        sync_scan_shlex = shlex.split(sync_scan_command) # prepare it
+
         # Use the top_port_parser
         output = self.run_command(sync_scan_shlex)
         xml_root = self.get_xml_et(output)
-        self.top_ports = self.filter_top_ports(xml_root)
+        self.top_ports = parser.filter_top_ports(xml_root)
         return self.top_ports
-        
-    def nmap_tcp_scan(self, host):
+
+    def nmap_tcp_scan(self, host, args=None):
         """
         Scan host using the nmap tcp connect
-        
+
         @cmd nmap -sT 192.168.178.1
         """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
         tcp_scan = " {host} {default}".format(host=host, default=self.tcp_connt)
         tcp_scan_command = self.default_command() + tcp_scan
-        tcp_scan_shlex = shlex.split(tcp_scan_command) # prepare it 
+        if(args):
+            tcp_scan_command += " {0}".format(args)
+        tcp_scan_shlex = shlex.split(tcp_scan_command) # prepare it
+
+        # Use the top_port_parser
+        output = self.run_command(tcp_scan_shlex)
+        xml_root = self.get_xml_et(output)
+        tcp_results = parser.filter_top_ports(xml_root)
+        return tcp_results
+        
+    def nmap_udp_scan(self, host, args=None):
+        """
+        Scan host using the nmap tcp connect
+
+        @cmd nmap -sU 192.168.178.1
+        """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
+        tcp_scan = " {host} {default}".format(host=host, default=self.udp_scan)
+        tcp_scan_command = self.default_command() + tcp_scan
+        if(args):
+            tcp_scan_command += " {0}".format(args)
+        tcp_scan_shlex = shlex.split(tcp_scan_command) # prepare it
         
         # Use the top_port_parser
         output = self.run_command(tcp_scan_shlex)
         xml_root = self.get_xml_et(output)
-        tcp_results = self.filter_top_ports(xml_root)
+        tcp_results = parser.filter_top_ports(xml_root)
         return tcp_results
-    
-    def nmap_ping_scan(self, host):
+
+    def nmap_ping_scan(self, host, args=None):
         """
         Scan host using nmaps' ping scan
-        
+
         @cmd nmap -sP 192.168.178.1
         """
         ping_scan = " {host} {default}".format(host=host, default=self.ping_scan)
         ping_scan_command = self.default_command() + ping_scan
-        ping_scan_shlex = shlex.split(ping_scan_command) # prepare it 
-        parser  = nmapparser.NmapCommandParser(None)
-        
+        if(args):
+            ping_scan_command += " {0}".format(args)
+        ping_scan_shlex = shlex.split(ping_scan_command) # prepare it
+        parser  = NmapCommandParser(None)
+
         output = self.run_command(ping_scan_shlex)
         xml_root = self.get_xml_et(output)
         ping_results = parser.parse_nmap_pingscan(xml_root)
-        return ping_results 
-    
-    def nmap_idle_scan(self, host):
+        return ping_results
+
+    def nmap_idle_scan(self, host, args=None):
         """
         Using nmap idle_scan
-        
+
         @cmd nmap -sL 192.168.178.1
         """
         idle_scan = " {host} {default}".format(host=host, default=self.idle_scan)
         idle_scan_command = self.default_command() + idle_scan
-        idle_scan_shlex = shlex.split(idle_scan_command) # prepare it 
-        parser  = nmapparser.NmapCommandParser(None)
-        
+        if(args):
+            idle_scan_command += " {0}".format(args)
+        idle_scan_shlex = shlex.split(idle_scan_command) # prepare it
+        parser  = NmapCommandParser(None)
+
         # Use the ping scan parser
         output = self.run_command(idle_scan_shlex)
         xml_root = self.get_xml_et(output)
         idle_results = parser.parse_nmap_pingscan(xml_root)
-        return idle_results 
-        
+        return idle_results
+
+class NmapHostDiscovery(Nmap):
+    """
+    This object will perform host discovery
+
+    1) Only port scan    (-Pn)
+    2) Only host discover    (-sn)
+    3) Arp discovery on a local network  (-PR)
+    4) Disable DNS resolution    (-n)
+    """
+    def __init__(self, path=None):
+        super(NmapHostDiscovery, self).__init__(path=path)
+
+        self.port_scan_only = "-Pn"
+        self.no_port_scan = "-sn"
+        self.arp_discovery = "-PR"
+        self.disable_dns = "-n"
+
+    def nmap_portscan_only(self, host, args=None):
+        """
+        Scan host using the nmap tcp connect
+
+        @cmd nmap -Pn 192.168.178.1
+        """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
+        scancommand = " {host} {default}".format(host=host, default=self.port_scan_only)
+        scan_command = self.default_command() + scancommand
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command) # prepare it
+
+        # Use the top_port_parser
+        output = self.run_command(scan_shlex)
+        xml_root = self.get_xml_et(output)
+        tcp_results = parser.filter_top_ports(xml_root)
+        return tcp_results
+
+    def nmap_no_portscan(self, host, args=None):
+        """
+        Scan host using the nmap tcp connect
+
+        @cmd nmap -sn 192.168.178.1
+        """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
+        scancommand = " {host} {default}".format(host=host, default=self.no_port_scan)
+        scan_command = self.default_command() + scancommand
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command) # prepare it
+
+        # Use the top_port_parser
+        output = self.run_command(scan_shlex)
+        xml_root = self.get_xml_et(output)
+        tcp_results = parser.parse_noportscan(xml_root)
+        return tcp_results
+
+    def nmap_arp_discovery(self, host, args=None):
+        """
+        Scan host using the nmap tcp connect
+
+        @cmd nmap -n 192.168.178.1
+        """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
+        scancommand = " {host} {default}".format(host=host, default=self.disable_dns)
+        scan_command = self.default_command() + scancommand
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command) # prepare it
+
+        # Use the top_port_parser
+        output = self.run_command(scan_shlex)
+        xml_root = self.get_xml_et(output)
+        tcp_results = parser.filter_top_ports(xml_root)
+        return tcp_results
+
+    def nmap_disable_dns(self, host, args=None):
+        """
+        Scan host using the nmap tcp connect
+
+        @cmd nmap -PR 192.168.178.1
+        """
+        parser  = NmapCommandParser(None)
+
+        if(args):
+            assert(isinstance(args, str)), "Expected string got {0} instead".format(type(args))
+
+        scancommand = " {host} {default}".format(host=host, default=self.arp_discovery)
+        scan_command = self.default_command() + scancommand
+        if(args):
+            scan_command += " {0}".format(args)
+        scan_shlex = shlex.split(scan_command) # prepare it
+
+        # Use the top_port_parser
+        output = self.run_command(scan_shlex)
+        xml_root = self.get_xml_et(output)
+        tcp_results = parser.filter_top_ports(xml_root)
+        return tcp_results
+
 if __name__=="__main__":
     parser = argparse.ArgumentParser(prog="Python3 nmap")
     parser.add_argument('-d', '--d', help='Help', required=True)
     args = parser.parse_args()
 
     nmap = NmapScanTechniques()
-    result = nmap.nmap_syn_scan(args.d)
+    result = nmap.nmap_udp_scan(args.d)
     print(json.dumps(result, indent=4, sort_keys=True))
